/*
 * asm1825 - Assembler for PIC16F1825 and others.
 *
 * $Id: lc.c,v 1.2 2017-08-12 00:30:26-04 andrew Exp andrew $
 * 
 * Copyright (c) 2017 Andrew Turnquist
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 * 
 */

#include "lc.h"
#include "psect.h"
#include "util.h"

int flatmodel = 1;	// Are we using a flat (no psects) model?
int lc;			// The (flat) location counter


void lcAdjustSize(void)
{
  /* lc is always relative to psect start. */
  if (curpsect->lc > curpsect->length) curpsect->length = curpsect->lc;
}


void lcSet(const int newlc)
{
  if (flatmodel) {
    lc = newlc;
  } else {
    (curpsect->lc) = newlc;
    lcAdjustSize();
  }
}


int lcGet(void)
{
  return (flatmodel ? lc : (curpsect->lc));
}


int lcInc(void)
{
  int oldlc = (flatmodel ? lc++ : (curpsect->lc)++);
  if (!flatmodel) lcAdjustSize();
  return oldlc;
}


int lcDec(void)
{
  int oldlc = (flatmodel ? lc-- : (curpsect->lc)--);
  if (!flatmodel) lcAdjustSize();
  return oldlc;
}


int lcGetAbs(void)
{
  if (flatmodel) return lc;
  if (curpsect->start < 0) {
    message(MSG_CRITICAL, "Attempt to get absolute lc value before psect start assigned for %s. Assembler error?", curpsect->pname);
    return curpsect->lc;
  }
  switch (curpsect->class) {
    case PS_CODE: return curpsect->start + curpsect->lc; break;
    case PS_EEDATA: return 0xf000 + curpsect->start + curpsect->lc; break;
    case PS_CONFIG: return 0x8000 + curpsect->start + curpsect->lc; break;
    case PS_COMMON: return curpsect->start + curpsect->lc; break;
    case PS_LINEAR: return curpsect->start + curpsect->lc; break;
    default:
      return curpsect->start + curpsect->lc;
  }
}

